From 338f296269cd41f25a57b4e37207f81553a320bb Mon Sep 17 00:00:00 2001
From: Yueyi Li <liyueyi@live.com>
Date: Tue, 14 Mar 2017 21:25:21 +0800
Subject: [PATCH] [ARM64][patch] Auto calculate kimage_voffset by kaslr offset

ARM64 kimage_voffset can be calculated if kernel ASLR offset is known.
Add a function to auto calculate kimage_voffset when '--kaslr=<offset>'
was set.
---
 arm64.c | 22 ++++++++++++++++++++++
 defs.h  |  1 +
 2 files changed, 23 insertions(+)

diff --git a/arm64.c b/arm64.c
index 6eaf96d..e2f128a 100644
--- a/arm64.c
+++ b/arm64.c
@@ -27,6 +27,7 @@
 static struct machine_specific arm64_machine_specific = { 0 };
 static int arm64_verify_symbol(const char *, ulong, char);
 static void arm64_parse_cmdline_args(void);
+static void arm64_calc_kimage_voffset(void);
 static void arm64_calc_phys_offset(void);
 static void arm64_calc_virtual_memory_ranges(void);
 static int arm64_kdump_phys_base(ulong *);
@@ -324,6 +325,9 @@ arm64_init(int when)
 		machdep->init_kernel_pgd = arm64_init_kernel_pgd;
 
 		/* use machdep parameters */
+		arm64_calc_kimage_voffset();
+
+		/* use machdep parameters */
 		arm64_calc_phys_offset();
 	
 		if (CRASHDEBUG(1)) {
@@ -726,6 +730,7 @@ arm64_parse_cmdline_args(void)
 				error(NOTE,
 					"setting kimage_voffset to: 0x%lx\n\n",
 					machdep->machspec->kimage_voffset);
+				machdep->flags |= KIMG_VOFFSET;
 				continue;
 			}
 
@@ -735,6 +740,23 @@ arm64_parse_cmdline_args(void)
 	}
 }
 
+static void
+arm64_calc_kimage_voffset(void)
+{
+	struct machine_specific *ms = machdep->machspec;
+	ulong kimage_offset;
+	ulong phys_offset;
+	struct syment *sp;
+
+	if (machdep->flags & KIMG_VOFFSET) /* --machdep override */
+		return;
+
+	if ((kt->flags2 & KASLR) && (kt->flags & RELOC_SET)){
+		arm_kdump_phys_base(&phys_offset);
+		ms->kimage_voffset = ms->vmalloc_start_addr + (kt->relocate * -1) - phys_offset;
+	}
+
+}
 
 static void
 arm64_calc_phys_offset(void)
diff --git a/defs.h b/defs.h
index edccdc7..c04588c 100644
--- a/defs.h
+++ b/defs.h
@@ -2989,6 +2989,7 @@ typedef signed int s32;
 #define IRQ_STACKS    (0x40)
 #define NEW_VMEMMAP   (0x80)
 #define VM_L4_4K      (0x100)
+#define KIMG_VOFFSET  (0x200)
 
 /*
  * Get kimage_voffset from /dev/crash
-- 
1.9.1

